

% This program registers the MRI datset to the OCT dataset (OCT = fixed,
% MRI = moving).


% Variables that must be present in workspace for code to run:
%   all semi quantitative and fully quantitative parameter maps
%   svOCT dataset (must be 6x6 mm field of view)
%   Brightfield image
%   tumor_mask



%% SECTION 1: Obtain MRI to Brightfield/Flourescence Registration Transformation

% 'Fixed' image set = Brightfield
% 'Moving' image set = MRI

close all

initial_directory = pwd;

%Load in the T2 weighted (T2w) images used for coregistration
for step = 1
    
    % Select the file containing the T2W images that will be used for
    % registration.

    fprintf('Select the file containg the T2W Images...\n')
    selpath = uigetdir;
    fprintf('%s\n',selpath);
    
    cd(selpath);
    
    % Specify up to 4 T2w images that will be used for registration. These
    % 4 images must have the fiducial markers clearly visible.
    
    T2_reg_1 = dicomread('t2w_10.dcm');
    T2_reg_1 = imresize(T2_reg_1,[64,64]);
    
    T2_reg_2 = dicomread('t2w_9.dcm');
    T2_reg_2 = imresize(T2_reg_2,[64,64]);
    
    T2_reg_3 = dicomread('t2w_8.dcm');
    T2_reg_3 = imresize(T2_reg_3,[64,64]);
    
    T2_reg_4 = dicomread('t2w_7.dcm');
    T2_reg_4 = imresize(T2_reg_4,[64,64]);
    
    
    % take the average of these 4 images along the depth dimension.
    T2_reg = zeros(64,64);
    for i = 1:1:64
        for j = 1:1:64
            T2_reg(i,j) = mean([T2_reg_1(i,j), T2_reg_2(i,j), T2_reg_3(i,j), T2_reg_4(i,j)]);
        end
    end
    
    cd(initial_directory)
    
end

% Obtain registration transformation for MRI to brightfield image
for step = 2
    
    fixed_BF = opt; %'opt' is the name of the optical brightfield image that must be loaded into the workspace before the code is run.
    
    moving = T2_reg./max(T2_reg(:)); % Normalized MR image to be used for registration
    
    % Perform control point selection with T2w image and brightfield image
    [mp,fp] = cpselect(moving,fixed_BF,'Wait',true);
    t_1 = fitgeotrans(mp,fp,'affine'); % Perform Affine point-based registration using the user-selected points
    Rfixed_BF_MRI = imref2d(size(fixed_BF));
    
end

% Assess acuracy of registration.
for step = 3
    
    registered_T2 = imwarp(moving,t_1,'OutputView',Rfixed_BF_MRI); % Register T2w image
    
    % Display registration results to user to assess accuracy
    figure, imshowpair(fixed_BF,registered_T2)
    figure, imshowpair(fixed_BF,registered_T2,'blend')
    figure, imshowpair(fixed_BF,registered_T2,'montage')
    
    user = input('Allignment ok (y/n)?','s');
    
    close all
    
    if user == 'n'
        error('Retry Allignment')
    end
    
end


%% SECTION 2: Obtain Brightfield/Flourescence to svOCT Registration Transformation

% Fixed = svOCT
% moving = Brightfield

% Obtain registration transformation for brightfield image to svOCT dataset.
for step = 1
    
    close all
    
    OCT_data_2D = SV_RGB; % 'SV_RGB' is the depth encoded svOCT dataset
    moving_BF = fixed_BF; % The 'fixed' brightfield dataset now becomes the 'moving' dataset.
    
    % Perform control point selection with brightfield/flourescence and svOCT dataset
    [mp,fp] = cpselect(moving_BF,OCT_data_2D,'Wait',true);
    t_2 = fitgeotrans(mp,fp,'similarity');
    Rfixed_BF_svOCT = imref2d(size(OCT_data_2D));
    
    % Register all images/parameter maps based on transformation
    
end
    
% Assess acuracy of registration
for step = 2

    registered_BF = imwarp(moving_BF,t_2,'OutputView',Rfixed_BF_svOCT);
    
    % Display results to user to assess accuracy of registration
    figure,imshowpair(OCT_data_2D,registered_BF,'blend')
    figure,imshowpair(AIP,registered_BF,"checkerboard")    

    user = input('Allignment ok (y/n)?','s');
    
    if user == 'n'
        error('Retry Allignment')
    end
    
    close all
    
end



%% SECTION 3: Register MRI Parameter Maps to svOCT Dataset

%NOTE: This section of the code registers two MRI parameter maps to the
%svOCT microvascular datset: k_trans, and the apparent diffusion
%coefficient (ADC). You may also add additional maps to this as long as
%they are intrinsically coregistered with the T2w MR-images from Section 1
%of this code.

% Perform MRI to brightfield registration for MRI parameter maps
for step = 1
    
    % for ADC parameter maps:
    ADC_1_reg1 = imwarp(ADC_1,t_1,'OutputView',Rfixed_BF_MRI); % Register slice 1 parameter maps
    ADC_2_reg1 = imwarp(ADC_2,t_1,'OutputView',Rfixed_BF_MRI); % Register slice 2 parameter maps
    
    % for k_trans parameter maps:
    k_trans_1_reg1 = imwarp(k_trans_map_1,t_1,'OutputView',Rfixed_BF_MRI); % Register slice 1 paramater maps
    k_trans_2_reg1 = imwarp(k_trans_map_2,t_1,'OutputView',Rfixed_BF_MRI); % Register slice 2 paramater maps
    
end

% Perform brightfield to svOCT registration for MRI parameter maps
for step = 2
    
    % for ADC parameter maps:
    ADC_1_reg2 = imwarp(ADC_1_reg1,t_2,'OutputView',Rfixed_BF_svOCT); % Register slice 1 paramater maps
    ADC_2_reg2 = imwarp(ADC_2_reg1,t_2,'OutputView',Rfixed_BF_svOCT); % Register slice 2 paramater maps
    
    % for k_trans parameter maps:
    k_trans_1_reg_2 = imwarp(k_trans_1_reg1,t_2,'OutputView',Rfixed_BF_svOCT); % Register slice 1 paramater maps
    k_trans_2_reg_2 = imwarp(k_trans_2_reg1,t_2,'OutputView',Rfixed_BF_svOCT); % Register slice 2 paramater maps
    
    
end

% Resize MR parameter maps to original resolution
for step = 3
    
    % for ADC parameter maps:
    registered_ADC_1 = imresize(ADC_1_reg2,[12,12]); % Slice 1
    registered_ADC_2 = imresize(ADC_2_reg2,[12,12]); % Slice 2
    
    % for k_trans parameter maps:
    registered_k_trans_1 = imresize(k_trans_1_reg_2,[12,12]); % Slice 1
    registered_k_trans_2 = imresize(k_trans_2_reg_2,[12,12]); % Slice 2
    
end

% Create depth averaged MRI dataset
for step = 4
    
    registered_ADC_avg = zeros(size(registered_k_trans_1,1),size(registered_k_trans_1,2));
    registered_k_trans_avg = zeros(size(registered_k_trans_1,1),size(registered_k_trans_1,2));
    
    for i = 1:1:size(registered_k_trans_1,1)
        for j = 1:1:size(registered_k_trans_1,2)
            registered_ADC_avg(i,j) = mean([registered_ADC_1(i,j), registered_ADC_2(i,j)]);
            registered_k_trans_avg(i,j) = mean([registered_k_trans_1(i,j), registered_k_trans_2(i,j)]);
        end
    end
    
end

%% SECTION 5: Segment Tumor

% Draw 2D tumor mask using brightfield image
for step = 1
    
    figure, imshow(registered_BF,[]);
    user_roi = drawpolygon;
    tumor_mask = createMask(user_roi);
    
end
    
% Decide which MRI voxels to exclude based on user defined overlap
% threshold
for step = 2
    
    overlap_threshold = 0.7; % Propotion of MR voxel within tumor mask that allows for a voxel to be assumed to be part of the tumor
    
    D1 = 64*ones(1,12);
    D2 = 64*ones(1,12);
    
    tumor_mask_cell = mat2cell(imresize(tumor_mask,[768,768]),D1,D2); % Create 12x12 cell array of tumor mask
    
    tumor_mask_MR = zeros(size(registered_k_trans_1,1),size(registered_k_trans_1,2));
    for i = 1:1:size(registered_k_trans_1,1)
        for j = 1:1:size(registered_k_trans_1,2)
            temp = tumor_mask_cell{i,j};
            if sum(temp(:))/(size(temp,1)*size(temp,2)) > overlap_threshold
                tumor_mask_MR(i,j) = 1;
            end
            
        end
    end
    
end

% Remove MRI voxels that are not contained in the tumor
for step = 3
    
    for i = 1:1:size(tumor_mask_MR,1)
        for j = 1:1:size(tumor_mask_MR,2)
            if tumor_mask_MR(i,j) == 0
                
                registered_ADC_1 = 0;
                registered_ADC_2 = 0;
                registered_ADC_avg(i,j) = 0;
                
                registered_k_trans_1(i,j) = 0;
                registered_k_trans_2(i,j) = 0;
                registered_k_trans_avg(i,j) = 0;
                
            end
        end
    end
    
end


%% Display Registered MRI parameter maps

close all

% Rename display maps
ADC_disp_1 = registered_ADC_1;
ADC_disp_2 = registered_ADC_2;
ADC_disp_avg = registered_ADC_avg;

k_trans_disp_1 = registered_k_trans_1;
k_trans_disp_2 = registered_k_trans_2;
k_trans_disp_avg = registered_k_trans_avg;

font_size = 15;

% Display slice 1:
for step = 1
    
    % ADC --------------------------------
    max_para = max(ADC_disp_1(:));
    min_para = min(nonzeros(ADC_disp_1(:)));
    figure, imagesc(ADC_disp_1,[min_para-(max_para-min_para)/250,max_para])
    title('ADC Slice 1','FontSize', font_size);
    cmap = jet(256);
    cmap(1,:) = 0; % Make lowest pixel black
    colormap(cmap);
    c = colorbar;
    c.Label.String = 'ADC [mm^2/Second]';
    c.Label.FontSize = font_size;
    c.FontSize = font_size;
    
    % K_trans --------------------------------
    n = 0;
    max_k_trans_1 = max(registered_k_trans_12(:));
    for i = 1:1:size(k_trans_disp_1,1)
        for j = 1:1:size(k_trans_disp_1,2)
            if tumor_mask_MR(i,j) == 1
                if k_trans_disp_1(i,j) == 0
                    k_trans_disp_1(i,j) = max_k_trans_1+max_k_trans_1/250;
                    n = n + 1;
                end
            end
        end
    end
    max_para = max(k_trans_disp_1(:));
    min_para = min(nonzeros(k_trans_disp_1(:)));
    figure, imagesc(k_trans_disp_1,[min_para-(max_para-min_para)/250,max_para])
    title('K_{trans} Slice 1','FontSize', font_size);
    cmap = jet(256);
    cmap(1,:) = 0; % Make lowest pixel black
    if n > 0 % Unphysiological pixels
        cmap(256,:) = 1; % Make largest pixel white
    end
    colormap(cmap);
    c = colorbar;
    c.Label.String = 'K_{trans} [min^{-1}]';
    c.Label.FontSize = font_size;
    c.FontSize = font_size;
    
end

% Display slice 2:
for step = 2
    
    % ADC --------------------------------
    max_para = max(ADC_disp_2(:));
    min_para = min(nonzeros(ADC_disp_2(:)));
    figure, imagesc(ADC_disp_2,[min_para-(max_para-min_para)/250,max_para])
    title('ADC Slice 2','FontSize', font_size);
    cmap = jet(256);
    cmap(1,:) = 0; % Make lowest pixel black
    colormap(cmap);
    c = colorbar;
    c.Label.String = 'ADC [mm^2/Second]';
    c.Label.FontSize = font_size;
    c.FontSize = font_size;
    
    % K_trans --------------------------------
    n = 0;
    max_k_trans_2 = max(registered_k_trans_2(:));
    for i = 1:1:size(k_trans_disp_2,1)
        for j = 1:1:size(k_trans_disp_2,2)
            if tumor_mask_MR(i,j) == 1
                if k_trans_disp_2(i,j) == 0
                    k_trans_disp_2(i,j) = max_k_trans_2+max_k_trans_2/250;
                    n = n + 1;
                end
            end
        end
    end
    max_para = max(k_trans_disp_2(:));
    min_para = min(nonzeros(k_trans_disp_2(:)));
    figure, imagesc(k_trans_disp_2,[min_para-(max_para-min_para)/250,max_para])
    title('K_{trans} Slice 2','FontSize', font_size);
    cmap = jet(256);
    cmap(1,:) = 0; % Make lowest pixel black
    if n > 0 % Unphysiological pixels
        cmap(256,:) = 1; % Make largest pixel white
    end
    colormap(cmap);
    c = colorbar;
    c.Label.String = 'K_{trans} [min^{-1}]';
    c.Label.FontSize = font_size;
    c.FontSize = font_size;
    
end

% Display depth-averaged slice:
for step = 3
    
    % ADC --------------------------------
    max_para = max(ADC_disp_avg(:));
    min_para = min(nonzeros(ADC_disp_avg(:)));
    figure, imagesc(ADC_disp_avg,[min_para-(max_para-min_para)/250,max_para])
    title('ADC','FontSize', font_size);
    cmap = jet(256);
    cmap(1,:) = 0; % Make lowest pixel black
    colormap(cmap);
    c = colorbar;
    c.Label.String = 'ADC [mm^2/Second]';
    c.Label.FontSize = font_size;
    c.FontSize = font_size;
    
    % K_trans --------------------------------
    n = 0;
    max_k_trans_avg = max(registered_k_trans_avg(:));
    for i = 1:1:size(k_trans_disp_avg,1)
        for j = 1:1:size(k_trans_disp_avg,2)
            if tumor_mask_MR(i,j) == 1
                if k_trans_disp_avg(i,j) == 0
                    k_trans_disp_avg(i,j) = max_k_trans_avg+max_k_trans_avg/250;
                    n = n + 1;
                end
            end
        end
    end
    max_para = max(k_trans_disp_avg(:));
    min_para = min(nonzeros(k_trans_disp_avg(:)));
    figure, imagesc(k_trans_disp_avg,[min_para-(max_para-min_para)/250,max_para])
    title('K_{trans}','FontSize', font_size);
    cmap = jet(256);
    cmap(1,:) = 0; % Make lowest pixel black
    if n > 0 % Unphysiological pixels
        cmap(256,:) = 1; % Make largest pixel white
    end
    colormap(cmap);
    c = colorbar;
    c.Label.String = 'K_{trans} [min^{-1}]';
    c.Label.FontSize = font_size;
    c.FontSize = font_size;
    
end

